@@ -1,6 +1,6 @@
 package HTML::Element;
 
-# $Id: Element.pm,v 1.37 1997/12/02 12:48:09 aas Exp $
+# $Id: Element.pm,v 1.34 1996/07/22 15:00:51 aas Exp $
 
 =head1 NAME
 
@@ -31,8 +31,6 @@ for a HTML document.
 
 The following methods are available:
 
-=over 4
-
 =cut
 
 
@@ -44,7 +42,7 @@ use vars qw($VERSION
 	    %emptyElement %optionalEndTag %linkElements %boolean_attr
            );
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.34 $ =~ /(\d+)\.(\d+)/);
 sub Version { $VERSION; }
 
 # Elements that does not have corresponding end tags (i.e. are empty)
@@ -52,7 +50,7 @@ sub Version { $VERSION; }
 			             img br hr wbr
 			             input area param
 			            );
-%optionalEndTag = map { $_ => 1 } qw(p li dt dd option); # th tr td);
+%optionalEndTag = map { $_ => 1 } qw(p li dt dd option th tr td);
 
 # Elements that might contain links and the name of the link attribute
 %linkElements =
@@ -88,7 +86,7 @@ sub Version { $VERSION; }
  ul     => 'compact',
 );
 
-=item $h = HTML::Element->new('tag', 'attrname' => 'value',...)
+=head2 $h = HTML::Element->new('tag', 'attrname' => 'value',...)
 
 The object constructor.  Takes a tag name as argument. Optionally,
 allows you to specify initial attributes at object creation time.
@@ -133,7 +131,7 @@ sub new
 
 
 
-=item $h->tag()
+=head2 $h->tag()
 
 Returns (optionally sets) the tag name for the element.  The tag is
 always converted to lower case.
@@ -152,7 +150,7 @@ sub tag
 
 
 
-=item $h->starttag()
+=head2 $h->starttag()
 
 Returns the complete start tag for the element.  Including leading
 "<", trailing ">" and attributes.
@@ -171,17 +169,8 @@ sub starttag
 	    exists($boolean_attr{$name}) && $boolean_attr{$name} eq $_) {
 	    $tag .= " \U$_";
 	} else {
-	    if ($val !~ /^\d+$/) {
-		# count number of " compared to number of '
-		if (($val =~ tr/\"/\"/) > ($val =~ tr/\'/\'/)) {
-		    # use single quotes around the attribute value
-		    HTML::Entities::encode_entities($val, "&'>");
-		    $val = qq('$val');
-		} else {
-		    HTML::Entities::encode_entities($val, '&">');
-		    $val = qq{"$val"};
-		}
-	    }
+	    HTML::Entities::encode_entities($val, '&">');
+	    $val = qq{"$val"} unless $val =~ /^\d+$/;
 	    $tag .= qq{ \U$_\E=$val};
 	}
     }
@@ -190,7 +179,7 @@ sub starttag
 
 
 
-=item $h->endtag()
+=head2 $h->endtag()
 
 Returns the complete end tag.  Includes leading "</" and the trailing
 ">".
@@ -204,7 +193,7 @@ sub endtag
 
 
 
-=item $h->parent([$newparent])
+=head2 $h->parent([$newparent])
 
 Returns (optionally sets) the parent for this element.
 
@@ -222,7 +211,7 @@ sub parent
 
 
 
-=item $h->implicit([$bool])
+=head2 $h->implicit([$bool])
 
 Returns (optionally sets) the implicit attribute.  This attribute is
 used to indicate that the element was not originally present in the
@@ -237,7 +226,7 @@ sub implicit
 
 
 
-=item $h->is_inside('tag',...)
+=head2 $h->is_inside('tag',...)
 
 Returns true if this tag is contained inside one of the specified tags.
 
@@ -259,7 +248,7 @@ sub is_inside
 
 
 
-=item $h->pos()
+=head2 $h->pos()
 
 Returns (and optionally sets) the current position.  The position is a
 reference to a HTML::Element object that is part of the tree that has
@@ -283,7 +272,7 @@ sub pos
 
 
 
-=item $h->attr('attr', [$value])
+=head2 $h->attr('attr', [$value])
 
 Returns (and optionally sets) the value of some attribute.
 
@@ -302,7 +291,7 @@ sub attr
 
 
 
-=item $h->content()
+=head2 $h->content()
 
 Returns the content of this element.  The content is represented as a
 reference to an array of text segments and references to other
@@ -317,7 +306,7 @@ sub content
 
 
 
-=item $h->is_empty()
+=head2 $h->is_empty()
 
 Returns true if there is no content.
 
@@ -331,7 +320,7 @@ sub is_empty
 
 
 
-=item $h->insert_element($element, $implicit)
+=head2 $h->insert_element($element, $implicit)
 
 Inserts a new element at current position and updates pos() to point
 to the inserted element.  Returns $element.
@@ -360,7 +349,7 @@ sub insert_element
 }
 
 
-=item $h->push_content($element_or_text,...)
+=head2 $h->push_content($element_or_text,...)
 
 Adds to the content of the element.  The content should be a text
 segment (scalar) or a reference to a HTML::Element object.
@@ -391,7 +380,7 @@ sub push_content
 
 
 
-=item $h->delete_content()
+=head2 $h->delete_content()
 
 Clears the content.
 
@@ -409,7 +398,7 @@ sub delete_content
 
 
 
-=item $h->delete()
+=head2 $h->delete()
 
 Frees memory associated with the element and all children.  This is
 needed because perl's reference counting does not work since we use
@@ -428,7 +417,7 @@ sub delete
 
 
 
-=item $h->traverse(\&callback, [$ignoretext])
+=head2 $h->traverse(\&callback, [$ignoretext])
 
 Traverse the element and all of its children.  For each node visited, the
 callback routine is called with the node, a startflag and the depth as
@@ -461,7 +450,7 @@ sub traverse
 
 
 
-=item $h->extract_links([@wantedTypes])
+=head2 $h->extract_links([@wantedTypes])
 
 Returns links found by traversing the element and all of its children.
 The return value is a reference to an array.  Each element of the
@@ -505,7 +494,7 @@ sub extract_links
 
 
 
-=item $h->dump()
+=head2 $h->dump()
 
 Prints the element and all its children to STDOUT.  Mainly useful for
 debugging.  The structure of the document is shown by indentation (no
@@ -531,7 +520,7 @@ sub dump
 
 
 
-=item $h->as_HTML()
+=head2 $h->as_HTML()
 
 Returns a string (the HTML document) that represents the element and
 its children.
@@ -577,7 +566,6 @@ sub format
 
 __END__
 
-=back
 
 =head1 BUGS
 
@@ -595,9 +583,13 @@ L<HTML::AsSubs>
 
 =head1 COPYRIGHT
 
-Copyright 1995-1997 Gisle Aas.
+Copyright 1995,1996 Gisle Aas.  All rights reserved.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
+=head1 AUTHOR
+
+Gisle Aas <aas@sn.no>
+
 =cut
