@@ -1,4 +1,4 @@
-# $Id: Protocol.pm,v 1.38 2001/04/21 03:56:03 gisle Exp $
+# $Id: Protocol.pm,v 1.23 1996/07/23 19:53:57 aas Exp $
 
 package LWP::Protocol;
 
@@ -6,12 +6,6 @@ package LWP::Protocol;
 
 LWP::Protocol - Base class for LWP protocols
 
-=head1 SYNOPSIS
-
- package LWP::Protocol::foo;
- require LWP::Protocol;
- @ISA=qw(LWP::Protocol);
-
 =head1 DESCRIPTION
 
 This class is used a the base class for all protocol implementations
@@ -28,9 +22,12 @@ method which is used to service a request. The overridden method can
 make use of the collect() function to collect together chunks of data
 as it is received.
 
-The following methods and functions are provided:
+=head1 SEE ALSO
 
-=over 4
+Inspect the F<LWP/Protocol/file.pm> and F<LWP/Protocol/http.pm> files
+for examples of usage.
+
+=head1 METHODS AND FUNCTIONS
 
 =cut
 
@@ -38,18 +35,16 @@ The following methods and functions are provided:
 
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.38 $ =~ /(\d+)\.(\d+)/);
 
 use strict;
 use Carp ();
-use HTTP::Status ();
-use HTTP::Response;
+use HTTP::Status 'RC_INTERNAL_SERVER_ERROR';
 require HTML::HeadParser;
 
 my %ImplementedBy = (); # scheme => classname
 
 
-=item $prot = LWP::Protocol->new()
+=head2 $prot = new HTTP::Protocol;
 
 The LWP::Protocol constructor is inherited by subclasses. As this is a
 virtual base class this method should B<not> be called directly.
@@ -58,22 +53,18 @@ virtual base class this method should B<not> be called directly.
 
 sub new
 {
-    my($class, $scheme, $ua) = @_;
+    my($class) = @_;
 
     my $self = bless {
-	scheme => $scheme,
-	ua => $ua,
-
-	# historical/redundant
-        parse_head => $ua->{parse_head},
-        max_size => $ua->{max_size},
+	'timeout' => 0,
+	'use_alarm' => 1,
+	'parse_head' => 1,
     }, $class;
-
     $self;
 }
 
 
-=item $prot = LWP::Protocol::create($scheme)
+=head2 $prot = LWP::Protocol::create($url)
 
 Create an object of the class implementing the protocol to handle the
 given scheme. This is a function, not a method. It is more an object
@@ -84,18 +75,17 @@ use to access protocols.
 
 sub create
 {
-    my($scheme, $ua) = @_;
+    my $scheme = shift;
     my $impclass = LWP::Protocol::implementor($scheme) or
 	Carp::croak("Protocol scheme '$scheme' is not supported");
 
     # hand-off to scheme specific implementation sub-class
-    my $protocol = $impclass->new($scheme, $ua);
-
-    return $protocol;
+    my $prot = new $impclass, $scheme;
+    return $prot;
 }
 
 
-=item $class = LWP::Protocol::implementor($scheme, [$class])
+=head2 $class = LWP::Protocol::implementor($scheme, [$class])
 
 Get and/or set implementor class for a scheme.  Returns '' if the
 specified scheme is not supported.
@@ -112,20 +102,17 @@ sub implementor
     my $ic = $ImplementedBy{$scheme};
     return $ic if $ic;
 
-    return '' unless $scheme =~ /^([.+\-\w]+)$/;  # check valid URL schemes
-    $scheme = $1; # untaint
-    $scheme =~ s/[.+\-]/_/g;  # make it a legal module name
-
     # scheme not yet known, look for a 'use'd implementation
     $ic = "LWP::Protocol::$scheme";  # default location
     $ic = "LWP::Protocol::nntp" if $scheme eq 'news'; #XXX ugly hack
-    no strict 'refs';
+    no strict qw(refs);
     # check we actually have one for the scheme:
-    unless (@{"${ic}::ISA"}) {
-	# try to autoload it
-	eval "require $ic";
+    unless (defined @{"${ic}::ISA"}) {
+	my $package = "$ic.pm";
+	$package =~ s|::|/|g;
+	eval { require "$package" };
 	if ($@) {
-	    if ($@ =~ /Can't locate/) { #' #emacs get confused by '
+	    if ($@ =~ /^Can't locate/) { #' #emacs get confused by '
 		$ic = '';
 	    } else {
 		die "$@\n";
@@ -137,7 +124,7 @@ sub implementor
 }
 
 
-=item $prot->request(...)
+=head2 $prot->request(...)
 
  $response = $protocol->request($request, $proxy, undef);
  $response = $protocol->request($request, $proxy, '/tmp/sss');
@@ -156,13 +143,29 @@ sub request
 }
 
 
-# legacy
-sub timeout    { shift->_elem('timeout',    @_); }
+=head2 $prot->timeout($seconds)
+
+Get and set the timeout value in seconds
+
+
+=head2 $prot->use_alarm($yesno)
+
+Indicates if the library is allowed to use the core alarm()
+function to implement timeouts.
+
+=head2 $prot->parse_head($yesno)
+
+Should we initialize response headers from the <head> section of HTML
+documents.
+
+=cut
+
+sub timeout  { shift->_elem('timeout',  @_); }
+sub use_alarm { shift->_elem('use_alarm', @_); }
 sub parse_head { shift->_elem('parse_head', @_); }
-sub max_size   { shift->_elem('max_size',   @_); }
 
 
-=item $prot->collect($arg, $response, $collector)
+=head2 $prot->collect($arg, $response, $collector)
 
 Called to collect the content of a request, and process it
 appropriately into a scalar, file, or by calling a callback.  If $arg
@@ -189,53 +192,40 @@ sub collect
 {
     my ($self, $arg, $response, $collector) = @_;
     my $content;
-    my($parse_head, $max_size) = @{$self}{qw(parse_head max_size)};
+    my($use_alarm, $parse_head, $timeout) =
+      @{$self}{qw(use_alarm parse_head timeout)};
 
     my $parser;
     if ($parse_head && $response->content_type eq 'text/html') {
 	$parser = HTML::HeadParser->new($response->{'_headers'});
     }
-    my $content_size = 0;
-
+    
     if (!defined($arg) || !$response->is_success) {
 	# scalar
 	while ($content = &$collector, length $$content) {
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
+	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
 	    $response->add_content($$content);
-	    $content_size += length($$content);
-	    if (defined($max_size) && $content_size > $max_size) {
-		LWP::Debug::debug("Aborting because size limit exceeded");
-		$response->push_header("Client-Aborted", "max_size");
-		#my $tot = $response->header("Content-Length") || 0;
-		#$response->header("X-Content-Range", "bytes 0-$content_size/$tot");
-		last;
-	    }
+	    alarm($timeout) if $use_alarm;
 	}
     }
     elsif (!ref($arg)) {
 	# filename
 	open(OUT, ">$arg") or
-	    return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-			  "Cannot write to '$arg': $!");
-        binmode(OUT);
+	    return new HTTP::Response RC_INTERNAL_SERVER_ERROR,
+			  "Cannot write to '$arg': $!";
         local($\) = ""; # ensure standard $OUTPUT_RECORD_SEPARATOR
 	while ($content = &$collector, length $$content) {
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
+	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
 	    print OUT $$content;
-	    $content_size += length($$content);
-	    if (defined($max_size) && $content_size > $max_size) {
-		LWP::Debug::debug("Aborting because size limit exceeded");
-		$response->push_header("Client-Aborted", "max_size");
-		#my $tot = $response->header("Content-Length") || 0;
-		#$response->header("X-Content-Range", "bytes 0-$content_size/$tot");
-		last;
-	    }
+	    alarm($timeout) if $use_alarm;
 	}
 	close(OUT);
     }
@@ -245,27 +235,28 @@ sub collect
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
+	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
             eval {
 		&$arg($$content, $response, $self);
 	    };
 	    if ($@) {
 	        chomp($@);
-		$response->push_header('X-Died' => $@);
-		$response->push_header("Client-Aborted", "die");
+		$response->header('X-Died' => $@);
 		last;
 	    }
+	    alarm($timeout) if $use_alarm
 	}
     }
     else {
-	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-				  "Unexpected collect argument  '$arg'");
+	return new HTTP::Response RC_INTERNAL_SERVER_ERROR,
+				  "Unexpected collect argument  '$arg'";
     }
     $response;
 }
 
 
-=item $prot->collect_once($arg, $response, $content)
+=head2 $prot->collect_once($arg, $response, $content)
 
 Can be called when the whole response content is available as
 $content.  This will invoke collect() with a collector callback that
@@ -286,17 +277,3 @@ sub collect_once
 }
 
 1;
-
-=head1 SEE ALSO
-
-Inspect the F<LWP/Protocol/file.pm> and F<LWP/Protocol/http.pm> files
-for examples of usage.
-
-=head1 COPYRIGHT
-
-Copyright 1995-2001 Gisle Aas.
-
-This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
-=cut
