@@ -1,5 +1,5 @@
 #
-# $Id: ftp.pm,v 1.30 2001/08/06 23:47:42 gisle Exp $
+# $Id: ftp.pm,v 1.16 1996/07/17 08:52:00 aas Exp $
 
 # Implementation of the ftp protocol (RFC 959). We let the Net::FTP
 # package do all the dirty work.
@@ -19,119 +19,12 @@ require LWP::Protocol;
 
 use strict;
 eval {
-    package LWP::Protocol::MyFTP;
-
     require Net::FTP;
     Net::FTP->require_version(2.00);
-
-    use vars qw(@ISA);
-    @ISA=qw(Net::FTP);
-
-    sub new {
-	my $class = shift;
-	LWP::Debug::trace('()');
-
-	my $self = $class->SUPER::new(@_) || return undef;
-
-	my $mess = $self->message;  # welcome message
-	LWP::Debug::debug($mess);
-	$mess =~ s|\n.*||s; # only first line left
-	$mess =~ s|\s*ready\.?$||;
-	# Make the version number more HTTP like
-	$mess =~ s|\s*\(Version\s*|/| and $mess =~ s|\)$||;
-	${*$self}{myftp_server} = $mess;
-	#$response->header("Server", $mess);
-
-	$self;
-    }
-
-    sub http_server {
-	my $self = shift;
-	${*$self}{myftp_server};
-    }
-
-    sub home {
-	my $self = shift;
-	my $old = ${*$self}{myftp_home};
-	if (@_) {
-	    ${*$self}{myftp_home} = shift;
-	}
-	$old;
-    }
-
-    sub go_home {
-	LWP::Debug::trace('');
-	my $self = shift;
-	$self->cwd(${*$self}{myftp_home});
-    }
-
-    sub request_count {
-	my $self = shift;
-	++${*$self}{myftp_reqcount};
-    }
-
-    sub ping {
-	LWP::Debug::trace('');
-	my $self = shift;
-	return $self->go_home;
-    }
-
 };
 my $init_failed = $@;
 
 
-sub _connect {
-    my($self, $host, $port, $user, $account, $password, $timeout) = @_;
-
-    my $key;
-    my $conn_cache = $self->{ua}{conn_cache};
-    if ($conn_cache) {
-	$key = "$host:$port:$user";
-	$key .= ":$account" if defined($account);
-	if (my $ftp = $conn_cache->withdraw("ftp", $key)) {
-	    if ($ftp->ping) {
-		LWP::Debug::debug('Reusing old connection');
-		# save it again
-		$conn_cache->deposit("ftp", $key, $ftp);
-		return $ftp;
-	    }
-	}
-    }
-
-    # try to make a connection
-    my $ftp = LWP::Protocol::MyFTP->new($host,
-					Port => $port,
-					Timeout => $timeout,
-				       );
-    # XXX Should be some what to pass on 'Passive' (header??)
-    unless ($ftp) {
-	$@ =~ s/^Net::FTP: //;
-	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR, $@);
-    }
-
-    LWP::Debug::debug("Logging in as $user (password $password)...");
-    unless ($ftp->login($user, $password, $account)) {
-	# Unauthorized.  Let's fake a RC_UNAUTHORIZED response
-	my $mess = scalar($ftp->message);
-	LWP::Debug::debug($mess);
-	$mess =~ s/\n$//;
-	my $res =  HTTP::Response->new(&HTTP::Status::RC_UNAUTHORIZED, $mess);
-	$res->header("Server", $ftp->http_server);
-	$res->header("WWW-Authenticate", qq(Basic Realm="FTP login"));
-	return $res;
-    }
-    LWP::Debug::debug($ftp->message);
-
-    my $home = $ftp->pwd;
-    LWP::Debug::debug("home: '$home'");
-    $ftp->home($home);
-
-    $conn_cache->deposit("ftp", $key, $ftp) if $conn_cache;
-
-    return $ftp;
-}
-
-
 sub request
 {
     my($self, $request, $proxy, $arg, $size, $timeout) = @_;
@@ -143,29 +36,29 @@ sub request
     # check proxy
     if (defined $proxy)
     {
-	return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-				   'You can not proxy through the ftp');
+	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+				  'You can not proxy through the ftp';
     }
 
     my $url = $request->url;
     if ($url->scheme ne 'ftp') {
 	my $scheme = $url->scheme;
-	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-		       "LWP::Protocol::ftp::request called for '$scheme'");
+	return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+		       "LWP::Protocol::ftp::request called for '$scheme'";
     }
 
     # check method
     my $method = $request->method;
 
     unless ($method eq 'GET' || $method eq 'HEAD' || $method eq 'PUT') {
-	return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-				   'Library does not allow method ' .
-				   "$method for 'ftp:' URLs");
+	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+				  'Library does not allow method ' .
+				  "$method for 'ftp:' URLs";
     }
 
     if ($init_failed) {
-	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-				   $init_failed);
+	return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+		       $init_failed;
     }
 
     my $host     = $url->host;
@@ -184,32 +77,40 @@ sub request
     }
 
     # We allow the account to be specified in the "Account" header
-    my $account = $request->header('Account');
-
-    my $ftp = $self->_connect($host, $port, $user, $account, $password, $timeout);
-    return $ftp if ref($ftp) eq "HTTP::Response"; # ugh!
+    my $acct     = $request->header('Account');
 
     # Create an initial response object
-    my $response = HTTP::Response->new(&HTTP::Status::RC_OK, "OK");
-    $response->header(Server => $ftp->http_server);
-    $response->header('Client-Request-Num' => $ftp->request_count);
-    $response->request($request);
+    my $response = new HTTP::Response &HTTP::Status::RC_OK, "Document follows";
+
+    my $ftp = new Net::FTP $host, Port => $port;
+    my $mess = $ftp->message;  # welcome message
+    LWP::Debug::debug($mess);
+    $mess =~ s|\n.*||s; # only first line left
+    $mess =~ s|\s*ready\.?$||;
+    # Make the version number more HTTP like
+    $mess =~ s|\s*\(Version\s*|/| and $mess =~ s|\)$||;
+    $response->header("Server", $mess);
+
+    $ftp->timeout($timeout) if $timeout;
+
+    LWP::Debug::debug("Logging in as $user (password $password)...");
+    unless ($ftp->login($user, $password, $acct)) {
+	# Unauthorized.  Let's fake a RC_UNAUTHORIZED response
+	my $res =  new HTTP::Response &HTTP::Status::RC_UNAUTHORIZED, $ftp->message;
+	$res->header("WWW-Authenticate", qq(Basic Realm="FTP login"));
+	return $res;
+    }
+    LWP::Debug::debug($ftp->message);
 
     # Get & fix the path
-    my @path =  grep { length } $url->path_segments;
+    my @path =  $url->path_components;
+    shift(@path);  # There will always be an empty first component
+    pop(@path) while @path && $path[-1] eq '';  # remove empty tailing comps
     my $remote_file = pop(@path);
     $remote_file = '' unless defined $remote_file;
 
-    my $type;
-    if (ref $remote_file) {
-	my @params;
-	($remote_file, @params) = @$remote_file;
-	for (@params) {
-	    $type = $_ if s/^type=//;
-	}
-    }
-
-    if ($type && $type eq 'a') {
+    my $params = $url->params;
+    if (defined($params) && $params eq 'type=a') {
 	$ftp->ascii;
     } else {
 	$ftp->binary;
@@ -218,32 +119,17 @@ sub request
     for (@path) {
 	LWP::Debug::debug("CWD $_");
 	unless ($ftp->cwd($_)) {
-	    return HTTP::Response->new(&HTTP::Status::RC_NOT_FOUND,
-				       "Can't chdir to $_");
+	    return new HTTP::Response &HTTP::Status::RC_NOT_FOUND,
+		       "Can't chdir to $_";
 	}
     }
 
     if ($method eq 'GET' || $method eq 'HEAD') {
-	LWP::Debug::debug("MDTM");
-	if (my $mod_time = $ftp->mdtm($remote_file)) {
-	    $response->last_modified($mod_time);
-	    if (my $ims = $request->if_modified_since) {
-		if ($mod_time <= $ims) {
-		    $response->code(&HTTP::Status::RC_NOT_MODIFIED);
-		    $response->message("Not modified");
-		    return $response;
-		}
-	    }
-	}
-
 	my $data;  # the data handle
 	LWP::Debug::debug("retrieve file?");
 	if (length($remote_file) and $data = $ftp->retr($remote_file)) {
-	    my($type, @enc) = LWP::MediaTypes::guess_media_type($remote_file);
-	    $response->header('Content-Type',   $type) if $type;
-	    for (@enc) {
-		$response->push_header('Content-Encoding', $_);
-	    }
+	    $response->header('Content-Type',
+			      LWP::MediaTypes::guess_media_type($remote_file));
 	    my $mess = $ftp->message;
 	    LWP::Debug::debug($mess);
 	    if ($mess =~ /\((\d+)\s+bytes\)/) {
@@ -270,8 +156,8 @@ sub request
 	    # 550 not a plain file, try to list instead
 	    if (length($remote_file) && !$ftp->cwd($remote_file)) {
 		LWP::Debug::debug("chdir before listing failed");
-		return HTTP::Response->new(&HTTP::Status::RC_NOT_FOUND,
-					   "File '$remote_file' not found");
+		return new HTTP::Response &HTTP::Status::RC_NOT_FOUND,
+		       "File '$remote_file' not found";
 	    }
 
 	    # It should now be safe to try to list the directory
@@ -291,14 +177,14 @@ sub request
 	    my $content = '';
 
 	    if (!defined($prefer)) {
-		return HTTP::Response->new(&HTTP::Status::RC_NOT_ACCEPTABLE,
-			       "Neither HTML nor directory listing wanted");
+		return new HTTP::Response &HTTP::Status::RC_NOT_ACCEPTABLE,
+				   "Neither HTML nor directory listing wanted";
 	    } elsif ($prefer eq 'html') {
 		$response->header('Content-Type' => 'text/html');
 		$content = "<HEAD><TITLE>File Listing</TITLE>\n";
 		my $base = $request->url->clone;
-		my $path = $base->path;
-		$base->path("$path/") unless $path =~ m|/$|;
+		my $path = $base->epath;
+		$base->epath("$path/") unless $path =~ m|/$|;
 		$content .= qq(<BASE HREF="$base">\n</HEAD>\n);
 		$content .= "<BODY>\n<UL>\n";
 		for (File::Listing::parse_dir(\@lsl, 'GMT')) {
@@ -319,8 +205,8 @@ sub request
 		$response = $self->collect_once($arg, $response, $content);
 	    }
 	} else {
-	    my $res = HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-			  "FTP return code " . $ftp->code);
+	    my $res = new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+			  "FTP return code " . $ftp->code;
 	    $res->content_type("text/plain");
 	    $res->content($ftp->message);
 	    return $res;
@@ -328,8 +214,8 @@ sub request
     } elsif ($method eq 'PUT') {
 	# method must be PUT
 	unless (length($remote_file)) {
-	    return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-				       "Must have a file name to PUT to");
+	    return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+				      "Must have a file name to PUT to";
 	}
 	my $data;
 	if ($data = $ftp->stor($remote_file)) {
@@ -339,11 +225,11 @@ sub request
 	    my $bytes = 0;
 	    if (defined $content) {
 		if (ref($content) eq 'SCALAR') {
-		    $bytes = $data->write($$content, length($$content));
+		    $bytes = $data->write($$content);
 		} elsif (ref($content) eq 'CODE') {
 		    my($buf, $n);
 		    while (length($buf = &$content)) {
-			$n = $data->write($buf, length($buf));
+			$n = $data->write($content);
 			last unless $n;
 			$bytes += $n;
 		    }
@@ -363,15 +249,15 @@ sub request
 	    $response->content("$bytes bytes stored as $remote_file on $host\n")
 
 	} else {
-	    my $res = HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-					  "FTP return code " . $ftp->code);
+	    my $res = new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+			  "FTP return code " . $ftp->code;
 	    $res->content_type("text/plain");
 	    $res->content($ftp->message);
 	    return $res;
 	}
     } else {
-	return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
-				   "Illegal method $method");
+	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+	       "Illegal method $method"
     }
 
     $response;
